---
title: drop_on
type: output
status: stable
categories: ["Utility"]
---

<!--
     THIS FILE IS AUTOGENERATED!

     To make changes please edit the contents of:
     lib/output/drop_on.go
-->

import Tabs from '@theme/Tabs';
import TabItem from '@theme/TabItem';

Attempts to write messages to a child output and if the write fails for one of a list of configurable reasons the message is dropped instead of being reattempted.

```yml
# Config fields, showing default values
output:
  label: ""
  drop_on:
    error: false
    back_pressure: ""
    output: {}
```

Regular Benthos outputs will apply back pressure when downstream services aren't accessible, and Benthos retries (or nacks) all messages that fail to be delivered. However, in some circumstances, or for certain output types, we instead might want to relax these mechanisms, which is when this output becomes useful.

## Fields

### `error`

Whether messages should be dropped when the child output returns an error. For example, this could be when an http_client output gets a 4XX response code.


Type: `bool`  
Default: `false`  

### `back_pressure`

An optional duration string that determines the maximum length of time to wait for a given message to be accepted by the child output before the message should be dropped instead. The most common reason for an output to block is when waiting for a lost connection to be re-established. Once a message has been dropped due to back pressure all subsequent messages are dropped immediately until the output is ready to process them again. Note that if `error` is set to `false` and this field is specified then messages dropped due to back pressure will return an error response.


Type: `string`  
Default: `""`  

```yml
# Examples

back_pressure: 30s

back_pressure: 1m
```

### `output`

A child output.


Type: `output`  
Default: `null`  

## Examples

<Tabs defaultValue="Dropping failed HTTP requests" values={[
{ label: 'Dropping failed HTTP requests', value: 'Dropping failed HTTP requests', },
{ label: 'Dropping from outputs that cannot connect', value: 'Dropping from outputs that cannot connect', },
]}>

<TabItem value="Dropping failed HTTP requests">

In this example we have a fan_out broker, where we guarantee delivery to our Kafka output, but drop messages if they fail our secondary HTTP client output.

```yaml
output:
  broker:
    pattern: fan_out
    outputs:
      - kafka:
          addresses: [ foobar:6379 ]
          topic: foo
      - drop_on:
          error: true
          output:
            http_client:
              url: http://example.com/foo/messages
              verb: POST
```

</TabItem>
<TabItem value="Dropping from outputs that cannot connect">

Most outputs that attempt to establish and long-lived connection will apply back-pressure when the connection is lost. The following example has a websocket output where if it takes longer than 10 seconds to establish a connection, or recover a lost one, pending messages are dropped.

```yaml
output:
  drop_on:
    back_pressure: 10s
    output:
      websocket:
        url: ws://example.com/foo/messages
```

</TabItem>
</Tabs>


